
interface PathConnection {
    fromModelUUID: string;
    fromUUID: string;
    toConnections: {
        toModelUUID: string;
        toUUID: string;
    }[];
}

interface ConnectionStore {
    connections: PathConnection[];
    setConnections: (connections: PathConnection[]) => void;
    addConnection: (newConnection: PathConnection) => void;
    removeConnection: (fromUUID: string, toUUID: string) => void;
}

interface ConveyorEventsSchema {
    modeluuid: string;
    modelName: string;
    type: "Conveyor";
    points: {
        uuid: string;
        position: [number, number, number];
        rotation: [number, number, number];
        actions: { uuid: string; name: string; type: string; material: string; delay: number | string; spawnInterval: number | string; isUsed: boolean; }[] | [];
        triggers: { uuid: string; name: string; type: string; isUsed: boolean; bufferTime: number; }[] | [];
        connections: {
            source: { modelUUID: string; pointUUID: string };
            targets: { modelUUID: string; pointUUID: string }[];
        };
    }[];
    position: [number, number, number];
    rotation: [number, number, number];
    speed: number | string;
}

interface VehicleEventsSchema {
    modeluuid: string;
    modelName: string;
    type: "Vehicle";
    points: {
        uuid: string;
        position: [number, number, number];
        rotation: [number, number, number];
        actions: { uuid: string; name: string; type: string; start: { x: number; y: number } | {}; hitCount: number; end: { x: number; y: number } | {}; buffer: number; };
        connections: {
            source: { modelUUID: string; pointUUID: string };
            targets: { modelUUID: string; pointUUID: string }[];
        };
        speed: number;
        isPlaying: boolean;
    };

    position: [number, number, number];
    rotation: [number, number, number];
}

interface StaticMachineEventsSchema {
    modeluuid: string;
    modelName: string;
    type: "StaticMachine";
    points: {
        uuid: string;
        position: [number, number, number];
        rotation: [number, number, number];
        actions: { uuid: string; name: string; buffer: number; material: string; };
        triggers: { uuid: string; name: string; type: string };
        connections: {
            source: { modelUUID: string; pointUUID: string };
            targets: { modelUUID: string; pointUUID: string }[];
        };
    };
    position: [number, number, number];
    rotation: [number, number, number];
}

interface ArmBotEventsSchema {
    modeluuid: string;
    modelName: string;
    type: "ArmBot";
    points: {
        uuid: string;
        position: [number, number, number];
        rotation: [number, number, number];
        actions: { uuid: string; name: string; speed: number; processes: { triggerId: string; startPoint: string; endPoint: string }[]; };
        triggers: { uuid: string; name: string; type: string };
        connections: {
            source: { modelUUID: string; pointUUID: string };
            targets: { modelUUID: string; pointUUID: string }[];
        };
    };
    position: [number, number, number];
    rotation: [number, number, number];
}

interface AssetEventSchema {
    modelUuid: string;
    modelName: string;
    position: [number, number, number];
    rotation: [number, number, number];
    state: "idle" | "running" | "stopped" | "disabled" | "error";
}

interface TriggerSchema {
    triggerUuid: string;
    triggerName: string;
    triggerType: "onComplete" | "onStart" | "onStop" | "delay" | "onError";
    delay: number;
    triggeredAsset: {
        triggeredModel: { modelName: string, modelUuid: string };
        triggeredAction: { actionName: string, actionUuid: string };
    } | null;
}

interface TransferPointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    actions: {
        actionUuid: string;
        actionName: string;
        actionType: "default" | "spawn" | "swap" | "despawn";
        material: string | "inherit";
        delay: number | "inherit";
        spawnInterval: number | "inherit";
        spawnCount: number | "inherit";
        triggers: TriggerSchema[] | [];
    }[];
}

interface VehiclePointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    actions: {
        actionUuid: string;
        actionName: string;
        actionType: "travel";
        material: string;
        unLoadDuration: number;
        loadCapacity: number;
        pickUpPoint: { x: number; y: number, z: number } | {};
        unLoadPoint: { x: number; y: number, z: number } | {};
        triggers: TriggerSchema[] | [];
    }[];
}

interface RoboticArmPointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    actions: {
        actionUuid: string;
        actionName: string;
        actionType: "pickAndPlace";
        process: { startPoint: string; endPoint: string };
        triggers: TriggerSchema[] | [];
    }[];
}

interface MachinePointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    actions: {
        actionUuid: string;
        actionName: string;
        actionType: "process";
        processTime: number;
        swapMaterial: string;
        triggers: TriggerSchema[] | [];
    }[];
}

interface StoragePointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    actions: {
        actionUuid: string;
        actionName: string;
        actionType: "storage";
        materials: { materialName: string; materialId: string; quantity: number }[];
        storageCapacity: number;
    }[];
}

interface TransferEventSchema extends AssetEventSchema {
    type: "transfer";
    speed: number;
    points: TransferPointSchema[];
}

interface VehicleSchemaEvent extends AssetEventSchema {
    type: "vehicle";
    speed: number;
    point: VehiclePointSchema;
}

interface RoboticArmSchemaEvent extends AssetEventSchema {
    type: "roboticArm";
    speed: number;
    point: RoboticArmPointSchema;
}

interface MachineSchemaEvent extends AssetEventSchema {
    type: "machine";
    point: MachinePointSchema;
}

interface StorageSchemaEvent extends AssetEventSchema {
    type: "storageUnit";
    point: StoragePointSchema;
}

type EventsSchema = TransferEventSchema | VehicleSchemaEvent | RoboticArmSchemaEvent | MachineSchemaEvent | StorageSchemaEvent | [];

type productsSchema = {
    productName: string;
    productId: string;
    eventsData: EventsSchema[];
}[] | []